#
# 1 - Read mssql db CA_UIM qos definitions via: 
#       SELECT origin, robot, probe, qos, source, target, r_table, h_table, table_id FROM S_QOS_DATA 
# 2 - Via input parameters in regex format we can match on several of these fields and decide what we want to delete.
# 3 - For each match we will generate:
#       DELETE from RN_QOS_DATA_0014 where table_id='5429'
#       DELETE from HN_QOS_DATA_0014 where table_id='5429'
#       DELETE from S_QOS_DATA where table_id ='5429'
#
# By DEFAULT we will run in simulation mode so you can experiment with the regex filters until it's ok
# 
# 
# input parameters:
# -d: output directory (default: c:\temp)
# -f: output file (default: report_qos_delete)
# -u: sql userid (default: mysql: root mssql: sa) if you use -u"trusted" we will use Windows trusted authentication
# -p: sql password *** required ***
# -r: repository/server name: default local host
# -y: 1: mssql 2: mysql (default: 1)
# -8: database name (default CA_UIM)
# -k: y: delete n: simulate (default: n)
# -v: y: run only a View report (default: n)
# -w: wait period in seconds after deleting -n qos entries (default: 0) if set to 0 => no wait
# -n: number of qos entries to be deleted before activating the -w seconds wait period (default 3, this corresponds to the rn, rh and qos itself)  
#
# --- this can activate the delete of older records
# -a: data older than xx dAys and matching all other regex filters will be deleted, this is specially interesting for data that is only needed in dashboard (only last poll is in most cases needed)
#     (in this case only raw data is removed)
#
# --- these are possible regex matches (Case Insensitive)
# -q: qos name include regex 
# -g: oriGin include regex (Attention, this will/can delete a lot)
# -o: prObe include regex 
# -b: robot include regex 
# -s: source include regex 
# -x: source (-s) match on ip (y,n) default y
# -t: target include regex 
#
# output:
# - print to output
#
# db: CA_UIM
#
# 09 July 2014 - Luc Christiaens
#
# Updates:
# 09/07/14: initial version
# 26/07/14: add -v to produce a qos overview report to help you deciding what qos entries to delete
#         : add the -w and -n options to wait xx seconds after deleting xx qos entries; this to give other Nimsoft sql operations to chance to continue to work normally
# 12/09/14: add -g to filter on origin
# 29/09/14: add option -8 to define database name (default: CA_UIM)
# 20/11/14: remove the "use" lines that are only there for perl2exe
# 04/12/14: add -a to remove all data older than xx days
# 25/09/15: add nimsoft_generic.pm to get common parameters (logon, servername...)
# 07/12/15: nimsoft_generic.dat will only use passwords that are crypted via nimsoft_crypt.pl
# 20/01/16: sql_type in nimsoft_generic
# 26/02/16: change sql query so that qos sources identified with ip are recognized by name (adaption thanks to sql query from jonhcw)
# 30/05/16: add b and d qos tables
# 05/06/16: print results of delete per qos
#

# --- Import external modules
use DBI;
use DBD::ODBC;
use Getopt::Std;
# crypt
use Crypt::RC4;
use MIME::Base64;
# UIM generic package
use nimsoft_generic;
# - change directory to directory where perl source is located to find our nimsoft_generic.dat file
use Cwd 'chdir';
use File::Basename;
chdir dirname $0;

# --- variables ---
$SERVER=$ENV{"computername"};
@hostnsplit=split(/\\/, $SERVER);
$hostnn=$#hostnsplit;
$hostn = $hostnsplit[$hostnn];
# - crypt key
$crkey       = "ab_def%hij&5";
$pgm="Nimsoft_Qos_Delete (v1.4)";

# --- get parameters
&getparams();

# --- open logging file 
open(OUTFILE1, ">$outfil.txt") || die("Cannot open file $outfil : $!");

# --- loop variables
$sqlcount=1;
$teller=0;
$tel=0;
$totcol=0;
$qoscount=0;

# --- prepare date
&calcdate;

# --- connect DB ---
if ($sqltype eq 'my')
  {
    $dbh = DBI->connect("DBI:mysql:database=$database;host=$sqlserver", "$sqluser", "$sqlpass") || die "Could not connect to nimsoftslm database on $sqlserver: $DBI::errstr";
  }
if ($sqltype eq 'ms')
  {
    if ($trusted eq 'yes')
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sqlserver;Database=$database;Trusted_Connection=Yes;") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
#        print "Using mssql trusted connection to connect to: $sqlserver\n";
      }
    else
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sqlserver;Database=$database;UID=$sqluser;PWD=$sqlpass") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
      }
  }

# --- did we request only a QoS overview report?
if ($only_view eq 'y')
{
$counto=1;
$countp=0;
$counto_total=0;
$count_sub=0;
print "\n";
print "Overview of Probe and #QoS\n";
print "==========================\n";
$sql = " select probe,de.name, source, COUNT(distinct target)'numtarget'
 from s_qos_definition de, s_qos_data da, cm_configuration_item_metric co, cm_configuration_item it, cm_device dv
 where de.qos_def_id = da.qos_def_id 
  and co.ci_id = it.ci_id 
 and it.dev_id = dv.dev_id 
 group by probe, de.name
, source order by probe, de.name,source";
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";
while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($probe, $qos, $source, $numtarget) = @row;
  if ($probe eq '')
   {
     $countp++;
   }
  $counto_total=$counto_total+$numtarget;
  if ($oprobe ne $probe && $oprobe ne '')
   {
     print "$counto - $oprobe: $count_sub\n";  
     $counto++;
     $count_sub=$numtarget; 
   }
  else
   {
     $count_sub=$count_sub+$numtarget; 
   }
  $oprobe=$probe;
 }
print "$counto - $oprobe: $count_sub\n";  
print"\n";
print "Total number of QoS entries found: $counto_total (excluding $countp QoS with no probe name)\n";
# --- disconnect ---
$dbh->disconnect();
# --- stop
exit; 
}


# --- sql query (in this case mysql and mssql are the same)
if ($filt_source eq '' || $filt_source_ip eq 'n')
  {
    $sql = "SELECT origin, robot, probe, qos, source, target, r_table, h_table, table_id FROM S_QOS_DATA";
  }
else
  {
#  new query to include sources with ip address
   $sql = "select qd.origin, qd.robot, qd.probe, qd.qos, cs.name, qd.target, qd.r_table, qd.h_table, qd.table_id,cs.ip from  CM_COMPUTER_SYSTEM cs,CM_DEVICE d,CM_CONFIGURATION_ITEM ci,CM_CONFIGURATION_ITEM_METRIC cim,S_QOS_DATA qd  where d.cs_id = cs.cs_id and ci.dev_id = d.dev_id and ci.dev_id = d.dev_id and cim.ci_id = ci.ci_id and qd.ci_metric_id = cim.ci_metric_id";
  }
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

print "Active regex filters:\n";
print "  - qos   : $filt_qos\n";
print "  - origin: $filt_origin\n"; 
print "  - probe : $filt_probe\n";
print "  - robot : $filt_robot\n";
print "  - source: $filt_source\n";
print "  - target: $filt_target\n";
print "Number of deletes: $del_num before wait period: $del_wait seconds\n";
print "\n";

$count9=1;

while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($qorigin,$qrobot,$qprobe,$qqos,$qsource,$qtarget,$qr_table,$qh_table,$qtable_id,$qip) = @row;
  $domatch=0;
# - match on filters
#   $domatch: 0 => no matching defined
#           : 1 => a filter was matched
#           : 2 => a filter did not match 
# once a filter has a $domatch=2 it will not be evaluated by others
#
    if ($filt_qos ne '')
     {
       if ($qqos =~ /$filt_qos/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
    if ($filt_origin ne '')
     {
       if ($qorigin =~ /$filt_origin/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
    if ($filt_probe ne '')
     {
       if ($qprobe =~ /$filt_probe/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_robot ne '')
     {
       if ($qrobot =~ /$filt_robot/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_source ne '')
     {
       if ($qsource =~ /$filt_source/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_target ne '')
     {
       if ($qtarget =~ /$filt_target/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

# - if there is a match add it in the table
    if ($domatch eq '1' || $domatch eq '0')
     {

# - put it in a table
       if ($dodelete ne 'y')
        {
           print "$count9: $qorigin $qrobot $qprobe $qqos $qsource $qtarget $qr_table $qh_table $qtable_id\n";
           print OUTFILE1 "Simulation $count9: $qorigin $qrobot $qprobe $qqos $qsource $qtarget $qr_table $qh_table $qtable_id\n";
        } 

# - put it in a table
       $qos9[$count9]="$qorigin\$qrobot\$qprobe\$qqos\$qsource\$qtarget\$qr_table\$qh_table\$qtable_id";
       $count9++;
     }
    $sqlcount++;
 }

# how many qos did we see ?
$count9=$count9-1;
print " \n";
print "Processed $sqlcount QoS definitions and selected $count9 records\n";

if ($dodelete ne 'y')
 {
   print "Stop due to simulation mode run\n";
   close (OUTFILE1);
   exit;
 }
 
print "Start deleting the matches\n";

# - we have all info, process the table and print all info with extra query to get qos details
$count9=1; 
$wait_count=0; 
while ($count9 < @qos9)
  {
     @numb9 = split (/\/, $qos9[$count9]); # create all keywords
     $yorigin=$numb9[0];
     $yrobot=$numb9[1];
     $yprobe=$numb9[2];
     $yqos = "$numb9[3]";
     $ysource = "$numb9[4]";
     $ytarget = "$numb9[5]";
     $yr_table = "$numb9[6]";
     $yh_table = "$numb9[7]";
     $ytable_id = "$numb9[8]";

# --- b&d tables are not referenced in s_qos_data, so create the table names ourself
     $yd_table = "DN_QOS_DATA_" . substr($yr_table, -4);
     $yb_table = "BN_QOS_DATA_" . substr($yr_table, -4);


# - delete selected qos entries
#       DELETE from 'RN_QOS_DATA_0014' where table_id='5429'
#       DELETE from 'HN_QOS_DATA_0014' where table_id='5429
#       DELETE from S_QOS_DATA where table_id ='5429'
#     if ($del_days eq '')
       if ($del_days ne '')
        {
           $addsql="and sampletime <= DATEADD(dd, -$del_days, GETDATE())";
        }
       else
        {
          $addsql=""; 
        } 
# - raw data
        $sql = "DELETE from $yr_table where table_id='$ytable_id' $addsql";
#        print "$count9: $sql\n";
#        print OUTFILE1 "$count9: $sql\n";
        $sth = $dbh->prepare($sql);
        $num1=$sth->execute or die "SQL qos detail Error: $DBI::errstr\n"; 
# - hourly history 
        $sql = "DELETE from $yh_table where table_id='$ytable_id' $addsql";
#        print "$count9: $sql\n";
#        print OUTFILE1 "$count9: $sql\n";
        $sth = $dbh->prepare($sql);
        $num2=$sth->execute or die "SQL qos detail Error: $DBI::errstr\n"; 
# - daily history 
        $sql = "DELETE from $yd_table where table_id='$ytable_id' $addsql";
#        print "$count9: $sql\n";
#        print OUTFILE1 "$count9: $sql\n";
        $sth = $dbh->prepare($sql);
        $num3=$sth->execute or die "SQL qos detail Error: $DBI::errstr\n"; 
# - baseline (no sampletime)
        $sql = "DELETE from $yb_table where table_id='$ytable_id'";
#        print "$count9: $sql\n";
#        print OUTFILE1 "$count9: $sql\n";
        $sth = $dbh->prepare($sql);
        $num4=$sth->execute or die "SQL qos detail Error: $DBI::errstr\n"; 
# - base entry
        $sql = "DELETE from S_QOS_DATA where table_id='$ytable_id' $addsql";
#        print "$count9: $sql\n";
#        print OUTFILE1 "$count9: $sql\n";
        $sth = $dbh->prepare($sql);
        $num5=$sth->execute or die "SQL qos detail Error: $DBI::errstr\n"; 
#      }
     if ($num1 eq '0E0') { $num1="0";}
     if ($num2 eq '0E0') { $num2="0";}
     if ($num3 eq '0E0') { $num3="0";}
     if ($num4 eq '0E0') { $num4="0";}
     if ($num5 eq '0E0') { $num5="0";}
      print "$count9: Deleted: RN: $num1 HN: $num2 DN: $num3 S_QOS_DATA: $num5 records ($yqos $ysource $ytarget)\n";
        print OUTFILE1 "$count9: Deleted: RN: $num1 HN: $num2 DN: $num3 S_QOS_DATA: $num5 records ($yqos $ysource $ytarget)\n";
#     else
#      {
#        $sql = "DELETE from $yr_table where table_id='$ytable_id' and sampletime <= DATEADD(dd, -$del_days, GETDATE())";
#        print "$count9: $sql\n";
#        print OUTFILE1 "$count9: $sql\n";
#        $sth = $dbh->prepare($sql);
#        $sth->execute or die "SQL qos detail Error: $DBI::errstr\n"; 
#      }

     $wait_count++;
     if (($wait_count eq $del_num) && ($del_wait ne "0"))
      {
        print "Starting sleep of $del_wait seconds (processed $count9 QoS)\n";
        sleep $del_wait;
        $wait_count=0;
      }
     $count9++;
 }

# --- put warning to restart data_engine
if ($count9 > 0)
 {
    print "\n";  
    print "Please restart probe data_engine BEFORE you recreate the same QoS metrics (please=you must)\n";
    print "\n";
 }

# --- disconnect ---
$dbh->disconnect();


# --- close HTML
close (OUTFILE1);


print "\n";
print "Created report: $outfil.txt\n";
print "\n";
exit 0;

# --- procedures


sub calcdate
 {
  $timeval = time();
  @timelist = gmtime ($timeval);
  if (@timelist[6] eq '0') { $dag = "Sunday" }
  if (@timelist[6] eq '1') { $dag = "Monday" }
  if (@timelist[6] eq '2') { $dag = "Tuesday" }
  if (@timelist[6] eq '3') { $dag = "Wednesday" }
  if (@timelist[6] eq '4') { $dag = "Thursday" }
  if (@timelist[6] eq '5') { $dag = "Friday" }
  if (@timelist[6] eq '6') { $dag = "Saturday" }
  if (@timelist[4] eq '0') { $maand = "January" }
  if (@timelist[4] eq '1') { $maand = "February" }
  if (@timelist[4] eq '2') { $maand = "March" }
  if (@timelist[4] eq '3') { $maand = "April" }
  if (@timelist[4] eq '4') { $maand = "May" }
  if (@timelist[4] eq '5') { $maand = "June" }
  if (@timelist[4] eq '6') { $maand = "July" }
  if (@timelist[4] eq '7') { $maand = "Augustus" }
  if (@timelist[4] eq '8') { $maand = "September" }
  if (@timelist[4] eq '9') { $maand = "October" }
  if (@timelist[4] eq '10') { $maand = "November" }
  if (@timelist[4] eq '11') { $maand = "December" }
  $maandnum = @timelist[4] +1;
  if ($maandnum < 10) {$maandnum="0$maandnum";}
  $dagnum = @timelist[3] ;
  if ($dagnum < 10) {$dagnum="0$dagnum";}
  $jaar = @timelist[5] + 1900;
#
# get time 
#
  $intim1 = time();
  $intim2 = localtime($intim1);
  @intime = split(/ +/ , $intim2);
  $intim = $intime[3]; 

# - format date to be used to compare with last poll (yyyy-mm-dd)
  $todayx="$jaar-$maandnum-$dagnum";


  print ("Today its $dag $dagnum $maand ($maandnum) $jaar $intim \n");
 }


sub getparams
{
  usage() unless @ARGV gt 0;
  my %Opts;
  getopts('d:f:u:p:r:y:q:k:o:b:s:t:v:w:n:g:8:a:x:',\%Opts);
  $outdir=$Opts{d};
  $outfile=$Opts{f};
  $sqluser=$Opts{u};
  $sqlpass=$Opts{p};
  $sqlopt=$Opts{y};
  $sqlserver=$Opts{r};
  $filt_qos=$Opts{q};
  $filt_probe=$Opts{o};
  $filt_robot=$Opts{b};
  $filt_source=$Opts{s};
  $filt_source_ip=$Opts{x};
  $filt_target=$Opts{t};
  $filt_origin=$Opts{g};
  $only_view=$Opts{v};
  $dodelete=$Opts{k};
  $del_wait=$Opts{w}; 
  $del_num=$Opts{n}; 
  $database=$Opts{8};
  $del_days=$Opts{a};

# --- get credentials and settings from common file (nimsoft_generic.dat)
($Server_With_Rest_Interface, $RestServicePort, $UIM_username, $UIM_password,$UIM_Domain,$UIM_Hub,$UIM_Robot,$sql_server,$sql_user,$sql_password,$sql_db,$sql_type) = nimsoft_generic::getRestData();

# -- de-crypt (need $encoded)
$decoded = decode_base64($UIM_password);
$UIM_password = RC4($crkey, $decoded);

$decoded = decode_base64($sql_password);
$sql_password = RC4($crkey, $decoded);

# --- if not used in parameters, use defaults from generic
if ($sqluser eq '') { $sqluser=$sql_user; }
if ($sqlpass eq '') { $sqlpass=$sql_password; }
if ($sqlserver eq '') { $sqlserver=$sql_server; }
if ($database eq '') { $database=$sql_db; }

# --- set defaults
  if ($sqlopt eq '1' || $sqlopt eq '')
   {
      $sqltype="ms";
   }
  if ($sqlopt eq '2')
   {
      $sqltype="my";
   }
  if ($sql_type eq 'mssql')
   {
      $sqltype="ms";
   } 
  if ($sql_type eq 'mysql')
   {
      $sqltype="my";
   } 
  if ($sqluser eq '')
   {
     if ($sqltype eq 'my')
      {
        $sqluser = "root";
      }
     if ($sqltype eq 'ms')
      {
        if ($sqlpass eq '')
         {
           $sqluser="trusted";
         }
        else
         {
           $sqluser = "sa";
         }
      }
   }
  if ($sqluser eq 'trusted')
      {
         $trusted=yes;
      }
  if ($sqlpass eq '')
   {
     if ($sqluser eq 'trusted')
      {
         $trusted=yes;
      }
     else
      {
        print "By default we try to access the local mysql with user: $sqluser\n";
        print "BUT you need at least the -p parameter to give a password\n";
        exit 12;
      }
   }

  if ($del_days eq '')
      {
#         print "All data for the selected records will be deleted (no date limit)\n";
      }
  else
      {
         print "Only data older than $del_days will be deleted\n";
      } 

  if ($database eq '')
   {
     $database="CA_UIM";
   }

  if ($sqlserver eq '')
   {
     $sqlserver=$hostn;
     if ($sqltype eq 'my')
      {
        print "We will connect the the mysql DB on server: $sqlserver\n";
      }
     if ($sqltype eq 'ms')
      {
        print "We will connect the the mssql DB on server: $sqlserver and userid: $sqluser\n";
      }
   }
  if ($outdir eq '')
   {
     $outdir = "c:\\temp";
   }
  if ($outfile eq '')
   {
     $outfile = "report\_nimsoft\_qos\_delete";
   }

  if ($dodelete eq '')
   {
     $dodelete="n";
     print "Running in Simulation mode, nothing will be deleted\n";
   }

  if ($only_view eq '')
   {
     $only_view="n";
   }
  if ($filt_source_ip eq '')
   {
     $filt_source_ip="y";
   }
  if ($only_view eq 'y')
   {
     print "Running only the QoS Overview report\n";
   }

  if ($del_wait eq '')
   {
     $del_wait="0";
   }

  if ($del_num eq '')
   {
     $del_num="3";
   }


# --- create dir + name
$outfil = "$outdir\\$outfile";
}

sub usage
 {
   print "      \n";
   print "nimsoft_qos_delete: delete QoS entries based on regex filters \n";
   print "   perl nimsoft_qos_delete.pl -k\"n\"\n";
   print "      \n";
   print "Input parameters:\n";
   print "  -d: output directory (default: c:\\temp)\n";
   print "  -f: output file (default: report_qos_delete.txt)\n";
#   print "  -u: sql userid (default: mysql: root mssql: sa)\n";
#   print "      if you use -u\"trusted\" we will use Windows trusted authentication\n";
#   print "  -p: sql password\n";
#   print "  -r: repository/server name: default local host\n";
#   print "  -y: 1: mssql 2: mysql (default: 1)\n";
#   print "  -8: database name, default: CA_UIM\n"; 
   print "  -k: y: delete n: simulate (default: n)\n";
   print "  -v: y: create only a qos overview report (default: n)\n";
   print "  -w: wait seconds between deletes, default 0\n";
   print "  -n: # of deletes before a wait period, default 3\n";
   print "Delete old data:\n";
   print "  -a: delete only data older than xx days (from RN_QOS tables)\n";
   print "Regex filters (Case Insensitive)\n";
   print "  -q: qos name include regex \n";
   print "  -g: origin include regex\n";
   print "  -o: probe include regex \n";
   print "  -b: robot include regex \n";
   print "  -s: source include regex \n";
   print "  -x: match source regex with matching ip (y,n) default y\n";
   print "  -t: target include regex \n";
   print "      \n";
   exit 0;
}

