#
# read mssql db CA_UIM
# 
# create HTML report with all QOS entries and list for each:
# - origin
# - robot
# - source 
# - probe
# - qos name
# - baseline defined?
# - target
# - oldest poll
# - latest poll
# - sample rate
# - min samplevalue
# - max samplevalue
# - samplevalue unit_short
# 
# input parameters:
# -d: output directory (default: c:\temp)
# -f: output file (default: report_nimsoft_sla)
# -u: sql userid (default: mysql: root mssql: sa) if you use -u"trusted" we will use Windows trusted authentication
# -p: sql password *** required ***
# -r: repository/server name: default local host
# -y: 1: mssql 2: mysql (default: 1)
# -w: detail report number of characters for target name (default: no limit) (when using the interface_traffic probe the target name can becoming long)
# -v: quick view: y: print on screen probes & number of qos, n: normal view, d: detail view (this will execute 1 query per qos, can be time consuming)
# -8: database name (default CA_UIM)
# -l: check if baseline definition exist (y,n) default n
#
# --- these are possible regex matches (Case Insensitive)
# -q: qos name include regex 
# -g: oriGin include regex (Attention, this will/can delete a lot)
# -o: prObe include regex 
# -b: robot include regex 
# -s: source include regex 
# -t: target include regex 
# -e: report only error detail records (y ro n) default: n
#
#
# output:
# - HTML report in directory -d and file -f (by default: c:\temp\report_qos_lastpoll.html)
#
# db: NimsoftSLM / CA_UIM
#
# 08 march 2014 - Luc Christiaens
#
# Updates:
# 09/04/14: updated for mysql+mssql
# 11/04/14: add option -y to make the selection mssql or mysql
# 30/04/14: if mssql and no userid/password, try windows integrated security
# 19/05/14: adapt so that user qos (that don't have a metric_id) will also be displayed
# 24/06/14: set some other columns also in red when date not today
# 03/07/14: add progress message for qos detail queries
# 04/07/14: create overview report
#         : add counter for each qos 
# 14/07/14: add delete file with delete statements for all 'inactive' qos metrics, up to you to copy interesting parts into sql quey
#           (but you can use: nimsoft_qos_delete.pl to delete with regex on several fields)
# 16/07/14: add extra view in overview report (source - probe)
# 27/07/14: add "with (NOLOCK)"
# 28/07/14: add probe overview
#         : add -v (comes from nimsoft_delete_qos.pl)
# 29/07/14: -v has also the options n & d to make it possible to give the reports without the time consuming individual qos queries (default: n)
# 29/08/14; counter total qos was incorrect
# 12/09/14: add overview on origin
# 14/09/14: add usage section if no params are used
# 26/09/14: add option -8 to define database name (default: CA_UIM)
# 05/10/14: adapt mysql for 8.0
# 31/01/15: add regex filters (-q, -g, -o, -b, -s and -t)
# 06/02/15: add option -e n: reports all records y: report only errors (default n)
# 07/02/15: add option -a : number of days since last poll to be reported as an error, default 3 
# 07/04/15: order by qos name
# 19/08/15: -v"n" will now also do the sql query to have min and max date, only not the count (because its the part that takes a lot of time)
# 25/09/15: add nimsoft_generic.pm to get common parameters (logon, servername...)
# 25/11/15: add counter to count generated delete statements
# 07/12/15: nimsoft_generic.dat will only use passwords that are crypted via nimsoft_crypt.pl
# 16/01/16: add -1: sort overview report by name (n) or digits (d) default: d
# 17/01/16: add -2: overview report count with ok/nok (d) or total (t) default: t
# 20/01/16: force the correct combination of -1 and -2
#         : add sql_type in nimsoft_generic
# 12/03/16: add -l to check if a baseline is defined (y, n) default n
#
# To do:
# - in print , if regex filters, total is total sql, not total selected
# - if filters, add these clearly in report
#
#

# --- Import external modules
use DBI;
use Getopt::Std;
use DBD::ODBC;
use PerlIO;
use Nimbus::API;
use Nimbus::Session;
use Nimbus::CFG;
use Nimbus::PDS;
# crypt
use Crypt::RC4;
use MIME::Base64;
# --- for time cals
use Time::Local;
# UIM generic package
use nimsoft_generic;
# - change directory to directory where perl source is located to find our nimsoft_generic.dat file
use Cwd 'chdir';
use File::Basename;
chdir dirname $0;

# --- variables ---
$SERVER=$ENV{"computername"};
@hostnsplit=split(/\\/, $SERVER);
$hostnn=$#hostnsplit;
$hostn = $hostnsplit[$hostnn];
$colspan=17;
$colspan1=3;
$countdel=0;
$countbase=0;
# - crypt key
$crkey       = "ab_def%hij&5";
$pgm="Nimsoft_Qos_Lastpoll (v1.3)";

# - these are needed for uim requests
$request_timeout = 30;
$request_retries = 3;

# --- get parameters
&getparams();

# --- loop variables
$sqlcount=0;
$teller=0;
$tel=0;
$totcol=0;
$qoscount=0;
$counterror=0;
# - table to store baseline metid
@tab2 = ();

# --- init overview report counters
%cqos=();
%cqos_probe=();
%cqos_source=();
%cprobe_source=();

# --- prepare date
&calcdate;

# --- connect DB ---
if ($sqltype eq 'my')
  {
    $dbh = DBI->connect("DBI:mysql:database=$database;host=$sqlserver", "$sqluser", "$sqlpass") || die "Could not connect to $database database on $sqlserver: $DBI::errstr";
  }
if ($sqltype eq 'ms')
  {
    if ($trusted eq 'yes')
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sqlserver;Database=$database;Trusted_Connection=Yes;") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
        print "Using mssql trusted connection to connect to: $sqlserver\n";
      }
    else
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sqlserver;Database=$database;UID=$sqluser;PWD=$sqlpass") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
      }
  }

# --- did we request only a QoS overview report?
if ($only_view eq 'y')
{
$counto=1;
$countp=0;
$counto_total=0;
$count_sub=0;
print "\n";
print "Overview of Probe and #QoS\n";
print "==========================\n";
$sql = " select probe,de.name, source, COUNT(distinct target)'numtarget'
 from s_qos_definition de, s_qos_data da, cm_configuration_item_metric co, cm_configuration_item it, cm_device dv
 where de.qos_def_id = da.qos_def_id 
  and co.ci_id = it.ci_id 
 and it.dev_id = dv.dev_id 
 group by probe, de.name
, source order by probe, de.name,source";
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";
while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($probe, $qos, $source, $numtarget) = @row;
  if ($probe eq '')
   {
     $countp++;
   }
  $counto_total=$counto_total+$numtarget;
#  print "$probe - $numtarget - $counto_total\n";
  if ($oprobe ne $probe && $oprobe ne '')
   {
     print "$counto - $oprobe: $count_sub\n";  
     $counto++;
     $count_sub=$numtarget; 
   }
  else
   {
     $count_sub=$count_sub+$numtarget; 
   }
  $oprobe=$probe;
 }
print "$counto - $oprobe: $count_sub\n";  
print"\n";
print "Total number of QoS entries found: $counto_total (excluding $countp QoS with no probe name)\n";
# --- disconnect ---
$dbh->disconnect();
# --- stop
exit; 
}

print "Active regex filters:\n";
print "  - qos   : $filt_qos\n";
print "  - origin: $filt_origin\n"; 
print "  - probe : $filt_probe\n";
print "  - robot : $filt_robot\n";
print "  - source: $filt_source\n";
print "  - target: $filt_target\n";
if ($error_only eq 'y')
 {
   print "  - report only error\n";
 }

# --- create uim defined baseline list (@tab2)
if ($report_baseline eq 'y')
 {
   &uim_baseline_list;
 }

# --- open HTML 
open(OUTFILE1, ">$outfil.html") || die("Cannot open file $outfil : $!");
open(OUTFILE2, ">$outfil\_overview.html") || die("Cannot open file $outfil\_overview : $!");
if ($delfile ne '')
 {
   open(DELFILE, ">$outfil\_delete.txt") || die("Cannot open file $outfil\_delete : $!");
 }
binmode OUTFILE2, ":encoding(UTF-8)";

# --- prepare HTML 
&report_header1;
&report_header2;

# --- sql query (nolock not in mysql)
if ($sqltype eq 'ms')
  {
    $sql = "select de.name, de.description , origin, robot, source, target , probe,  de.unit_short, de.r_table, de.h_table, samplerate, table_id,da.ci_metric_id 
 from s_qos_definition de with (NOLOCK), s_qos_data da with (NOLOCK), cm_configuration_item_metric co with (NOLOCK), cm_configuration_item it with (NOLOCK), cm_device dv
 with (NOLOCK) where de.qos_def_id = da.qos_def_id 
  and co.ci_id = it.ci_id 
 and it.dev_id = dv.dev_id 
 group by table_id,de.name, de.description , origin, robot, source, target , probe,  de.unit_short, de.r_table, de.h_table, samplerate,da.ci_metric_id order by de.name";
  }
else
  {
    $sql = "select de.name, de.description , origin, robot, source, target , probe,  de.unit_short, de.r_table, de.h_table, samplerate, table_id 
 from $database.s_qos_definition de, $database.s_qos_data da, $database.cm_configuration_item_metric co, $database.cm_configuration_item it, $database.cm_device dv
 where de.qos_def_id = da.qos_def_id 
  and co.ci_id = it.ci_id 
 and it.dev_id = dv.dev_id 
 group by table_id,de.name, de.description , origin, robot, source, target , probe,  de.unit_short, de.r_table, de.h_table, samplerate";
  }
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

$count9=0;
$counta=0;
while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($qqos,$qdesc,$qorigin,$qrobot,$qsource,$qtarget,$qprobe,$qunit,$qr_table, $qh_table, $qsamplerate,$qtable_id, $qmetid) = @row;

# 31/01 regex filter
  $domatch=0;
# - match on filters
#   $domatch: 0 => no matching defined
#           : 1 => a filter was matched
#           : 2 => a filter did not match 
# once a filter has a $domatch=2 it will not be evaluated by others
#
    if ($filt_qos ne '')
     {
       if ($qqos =~ /$filt_qos/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
    if ($filt_origin ne '')
     {
       if ($qorigin =~ /$filt_origin/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }
    if ($filt_probe ne '')
     {
       if ($qprobe =~ /$filt_probe/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_robot ne '')
     {
       if ($qrobot =~ /$filt_robot/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_source ne '')
     {
       if ($qsource =~ /$filt_source/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

    if ($filt_target ne '')
     {
       if ($qtarget =~ /$filt_target/i && $domatch ne '2') 
         {
           $domatch=1;
         }
       else
         {
           $domatch=2;
         }
     }

# - if there is a match add it in the table
    if ($domatch eq '1' || $domatch eq '0')
     {
       $dobase="";
# - do we have a baseline defined.
       if ($report_baseline eq 'y')
        {
          if( exists($tab2{$qmetid}) && $qmetid ne '' )
           {
             $dobase="y";
             $countbase++;
#             print "ok $qqos $dobase $qmetid\n";
           }
          else
           {
             $dobase="-";
#             print "   nok $qqos $dobase $qmetid\n";
           } 
         }
# - put it in a table
       $count9++;
       $qos9[$count9]="$qqos\$qdesc\$qrobot\$qsource\$qtarget\$qprobe\$qunit\$qr_table\$qsamplerate\$qtable_id\$qh_table\$qorigin\$dobase";

     }
# 31/01 end regex filters
   $sqlcount++;
 }

# how many qos did we see ?
  print "Processed $sqlcount QoS definitions, selected $count9 QoS definitions\n";
  if ($only_view eq 'd')
   {
     print "Start queries to get detail for each QoS\n";
   }
  else
   {
     print "Report will NOT have the QoS details due to parameter -v not set to value d\n";
   } 

# - we have all info, process the table and print all info with extra query to get qos details
  $count9=1;  
  while ($count9 < @qos9)
    {
     @numb9 = split (/\/, $qos9[$count9]); # create all keywords
     $qname=$numb9[0];
     $qdesc=$numb9[1];
     $qrobot=$numb9[2];
     $qsource = "$numb9[3]";
     $qtarget = "$numb9[4]";
     $qprobe = "$numb9[5]";
     $qunit = "$numb9[6]";
     $qr_table = "$numb9[7]";
     $qsamplerate = "$numb9[8]";
     $qtable_id = "$numb9[9]";
     $qh_table = "$numb9[10]";
     $qorigin = "$numb9[11]";
     $qbaseline = "$numb9[12]";
     $count9++;

# - get detail qos data (only if -v"d")
    if ($only_view eq 'd')
     {
       if ($sqltype eq 'ms')
        {
          $sql = "select count(*),MIN(sampletime), MAX(sampletime), min(samplevalue), MAX(samplevalue), AVG(samplevalue) from $qr_table with (NOLOCK) where table_id = '$qtable_id'";
        }
       else
        {
          $sql = "select count(*),MIN(sampletime), MAX(sampletime), min(samplevalue), MAX(samplevalue), AVG(samplevalue) from $qr_table where table_id = '$qtable_id'";
        } 
       $sth = $dbh->prepare($sql);
       $sth->execute or die "SQL qos detail Error: $DBI::errstr\n"; 

       while (@row = $sth->fetchrow_array) 
        {
# - decode line
          ($qcount,$qold_poll,$qlast_poll,$qmin_sample,$qmax_sample,$qavg_sample) = @row;
# - play with date & time format
          $qodate=substr($qold_poll,0,10);
          $qldate=substr($qlast_poll,0,10);
          $qotime=substr($qold_poll,10,6);
          $qltime=substr($qlast_poll,10,6);
        }  

# - generate message for each 100 qos
      $counta++; 
      if ($counta =~ /00$/)
       {
         print "  Queried $counta QoS details with count (total: $sqlcount)\n";
       }
     }
    else
     {
       if ($sqltype eq 'ms')
        {
          $sql = "select MIN(sampletime), MAX(sampletime) from $qr_table with (NOLOCK) where table_id = '$qtable_id'";
        }
       else
        {
          $sql = "select MIN(sampletime), MAX(sampletime) from $qr_table where table_id = '$qtable_id'";
        } 
       $sth = $dbh->prepare($sql);
       $sth->execute or die "SQL qos detail Error: $DBI::errstr\n"; 

       while (@row = $sth->fetchrow_array) 
        {
# - decode line
          ($qold_poll,$qlast_poll) = @row;
# - play with date & time format
          $qodate=substr($qold_poll,0,10);
          $qldate=substr($qlast_poll,0,10);
          $qotime=substr($qold_poll,10,6);
          $qltime=substr($qlast_poll,10,6);
          $qmin_sample="-";
          $qmax_sample="-";
          $qavg_sample="-";
        }  

# - generate message for each 100 qos
      $counta++; 
      if ($counta =~ /00$/)
       {
         print "  Queried $counta QoS details without count (total: $sqlcount)\n";
       }
     } 
# --- overview counters
# - does counter exist?
    $key10="$qsource\_ok";
    if (exists($csource{$key10}))
     {
#       print "csource $key10 found: $csource{$key10} \n";
     }
    else
     {
#       print "csource $key10 empty: $csource{$key10} \n";
       $csource{"$qsource\_ok"} = 0;
       $csource{"$qsource\_nok"} = 0;
     }

# - does counter exist?
    $key10="$qname\_ok";
    if (exists($cqname{$key10}))
     {
#       print "cqname $key10 found: $cqname{$key10} \n";
     }
    else
     {
#       print "cqname $key10 empty: $cqname{$key10} \n";
       $cqname{"$qname\_ok"} = 0;
       $cqname{"$qname\_nok"} = 0;
     }

    $key10="$qprobe\_$qname\_ok";
    if (exists($cprobe_qos{$key10}))
     {
#       print "cprobe_qos $key10 found: $cprobe_qos{$key10} \n";
     }
    else
     {
#       print "cprobe_qos $key10 empty: $cprobe_qos{$key10} \n";
       $cprobe_qos{"$qprobe\_$qname\_ok"} = 0;
       $cprobe_qos{"$qprobe\_$qname\_nok"} = 0;
     }

    $key10="$qprobe\_ok";
    if (exists($cprobe{$key10}))
     {
#       print "cprobe $key10 found: $cprobe{$key10} \n";
     }
    else
     {
#       print "cprobe $key10 empty: $cprobe{$key10} \n";
       $cprobe{"$qprobe\_ok"} = 0;
       $cprobe{"$qprobe\_nok"} = 0;
     }

    $key10="$qname\_$qsource\_ok";
    if (exists($cqos_source{$key10}))
     {
#       print "cqos_source $key10 found: $cqos_source{$key10} \n";
     }
    else
     {
#       print "cqos_source $key10 empty: $cqos_source{$key10} \n";
       $cqos_source{"$qname\_$qsource\_ok"} = 0;
       $cqos_source{"$qname\_$qsource\_nok"} = 0;
     }

    $key10="$qprobe\_$qsource\_ok";
    if (exists($cprobe_source{$key10}))
     {
#       print "cprobe_source $key10 found: $cprobe_source{$key10} \n";
     }
    else
     {
#       print "cprobe_source $key10 empty: $cprobe_source{$key10} \n";
       $cprobe_source{"$qprobe\_$qsource\_ok"} = 0;
       $cprobe_source{"$qprobe\_$qsource\_nok"} = 0;
     }

    $key10="$qsource\_$qprobe\_ok";
    if (exists($csource_probe{$key10}))
     {
#       print "csource_probe $key10 found: $csource_probe{$key10} \n";
     }
    else
     {
#       print "csource_probe $key10 empty: $csource_probe{$key10} \n";
       $csource_probe{"$qsource\_$qprobe\_ok"} = 0;
       $csource_probe{"$qsource\_$qprobe\_nok"} = 0;
     }

    $key10="$qorigin\_ok";
    if (exists($corigin{$key10}))
     {
#       print "corigin $key10 found: $corigin{$key10} \n";
     }
    else
     {
#       print "corigin $key10 empty: $corigin{$key10} \n";
       $corigin{"$qorigin\_ok"} = 0;
       $corigin{"$qorigin\_nok"} = 0;
     }

# - end init counters
#     print "process $counta qos: $qname source: $qsource target: $qtarget \n";

    if ($todayx eq $qldate || $overview_sort eq 'd' || $overview_count eq 't')
     {
       $csource{"$qsource\_ok"} += 1;
       $cqname{"$qname\_ok"} += 1;
       $corigin{"$qorigin\_ok"} += 1;
       $cprobe_qos{"$qprobe\_$qname\_ok"} += 1;
       $cprobe{"$qprobe\_ok"} += 1;
       $cqos_source{"$qname\_$qsource\_ok"} += 1;
       $cprobe_source{"$qprobe\_$qsource\_ok"} += 1;
       $csource_probe{"$qsource\_$qprobe\_ok"} += 1;
     }
    else
     { 
       $csource{"$qsource\_nok"} += 1;
       $cqname{"$qname\_nok"} += 1;
       $corigin{"$qorigin\_nok"} += 1;
       $cprobe_qos{"$qprobe\_$qname\_nok"} += 1;
       $cprobe{"$qprobe\_nok"} += 1;
       $cqos_source{"$qname\_$qsource\_nok"} += 1;
       $cprobe_source{"$qprobe\_$qsource\_nok"} += 1;
       $csource_probe{"$qsource\_$qprobe\_nok"} += 1;
     }

# -- debug
#    if ($qprobe =~ /interface\_traffic/i)
#     {
#      $key2="$qprobe\_$qsource\_ok";
#      $key3="$qprobe\_$qsource\_nok";
#      print "$qrobot $qsource $qprobe ok: $cprobe_source{$key2} nok: $cprobe_source{$key3} $todayx $qldate\n";
#     }

# - print QOS detail
     &report_detail;
# - repeat title every xx lines

     if ($totcol > 30)
      {
        &report_header2();
        $totcol=0;
      } 
   }

# --- disconnect ---
$dbh->disconnect();

# --- list overview counters
# - prepare HTML 
&report_over_header1;

# --- overview origin
$pcount=1;
$overtit="Origin";
&report_over_header2;
if ($overview_sort eq 'd')
 {
   foreach $sentry (sort sort_corigin(%corigin))
   {
     &sorigin;
   }
 }
else
 {
   foreach $sentry (sort keys(%corigin))
    {
       &sorigin;
    }
 }
&report_emptyline_over;

sub sorigin
{
   $over1=$sentry;
   $over2=$corigin{$sentry};
   $over0="Origin $pcount";
   if ($sentry =~ /_ok/i)
    {
# save ok into $ook
      $ook=$over2;
# save nok into $onok
      $onok=$oval; 
      $over1=~ s/\_ok//g ;
      &report_over_detail(); 
      $pcount++;
      $ocount++;
    }
   else
    {
      $oval=$over2;
    }
}

sub sort_corigin
{
  $corigin{$a} <=> $corigin{$b};
}


# --- overview qos name
$pcount=1;
$overtit="QOS";
&report_over_header2;
if ($overview_sort eq 'd')
 {
   foreach $sentry (sort sort_cqname(%cqname))
   {
     &sqname;
   }
 }
else
 {
  foreach $sentry (sort keys(%cqname))
   {
     &sqname;
   }
 }
&report_emptyline_over;

sub sqname
{
   $over1=$sentry;
   $over2=$cqname{$sentry};
   $over0="qos $pcount";
   if ($sentry =~ /_ok/i)
    {
# save ok into $ook
      $ook=$over2;
# save nok into $onok
      $onok=$oval; 
      $over1=~ s/\_ok//g ;
      &report_over_detail(); 
      $pcount++;
      $ocount++;
    }
   else
    {
      $oval=$over2;
    }
}

sub sort_cqname
{
  $cqname{$a} <=> $cqname{$b};
}

# --- overview source
$pcount=1;
$overtit="Source";
&report_over_header2;
if ($overview_sort eq 'd')
 {
   foreach $sentry (sort sort_csource keys(%csource))
   {
     &ssource;
   }
 }
else
 {
   foreach $sentry (sort keys(%csource))
    {
      &ssource;
    }
 }
&report_emptyline_over;

sub ssource
{
   $over1=$sentry;
   $over2=$csource{$sentry};
   $over0="Source $pcount";
   if ($sentry =~ /_ok/i)
    {
# save ok into $ook
      $ook=$over2;
# save nok into $onok
      $onok=$oval; 
      $over1=~ s/\_ok//g ;
      &report_over_detail(); 
      $pcount++;
      $ocount++;
    }
   else
    {
      $oval=$over2;
    }
}

sub sort_csource
{
  $csource{$a} <=> $csource{$b};
}


# --- overview probe 
$pcount=1;
$overtit="Probe";
&report_over_header2;
if ($overview_sort eq 'd')
 {
   foreach $psentry (sort sort_cprobe keys(%cprobe))
   {
     &sprobe;
   }
 }
else
 {
   foreach $psentry (sort keys(%cprobe))
    {
      &sprobe;
    }
 }
&report_emptyline_over;

sub sprobe
{
   $over1=$psentry;
   $over2=$cprobe{$psentry};
   $over0="Probe $pcount";
   if ($psentry =~ /_ok/i)
    {
      $ook=$over2;
      $onok=$oval;
      $over1=~ s/\_ok//g ;
      &report_over_detail(); 
      $pcount++;
      $ocount++;
    }
   else
    {
      $oval=$over2;
    }
}

sub sort_cprobe
{
  $cprobe{$a} <=> $cprobe{$b};
}

# --- overview probe and source
$pcount=1;
$overtit="Probe and Source";
&report_over_header2;
if ($overview_sort eq 'd')
 {
   foreach $psentry (sort sort_cprobe_source keys(%cprobe_source))
   {
     &sprobe_source;
   }
 }
else
 {
   foreach $psentry (sort keys(%cprobe_source))
    {
      &sprobe_source;
    }
 }
&report_emptyline_over;

sub sprobe_source
{
   $over1=$psentry;
   $over2=$cprobe_source{$psentry};
   $over0="Probe_Source $pcount";
   if ($psentry =~ /_ok/i)
    {
      $ook=$over2;
      $onok=$oval;
      $over1=~ s/\_ok//g ;
      &report_over_detail(); 
      $pcount++;
      $ocount++;
    }
   else
    {
      $oval=$over2;
    }
}

sub sort_cprobe_source
{
  $cprobe_source{$a} <=> $cprobe_source{$b};
}

# --- overview source and probe
$pcount=1;
$overtit="Source and Probe";
&report_over_header2;
if ($overview_sort eq 'd')
 {
   foreach $psentry (sort sort_csource_probe keys(%csource_probe))
   {
     &ssource_probe;
   }
 }
else
 {
   foreach $psentry (sort keys(%csource_probe))
    {
      &ssource_probe;
    }
 }
&report_emptyline_over;

sub ssource_probe
{
   $over1=$psentry;
   $over2=$csource_probe{$psentry};
   $over0="Source_Probe $pcount";
   if ($psentry =~ /_ok/i)
    {
      $ook=$over2;
      $onok=$oval;
      $over1=~ s/\_ok//g ;
      &report_over_detail(); 
      $pcount++;
      $ocount++;
    }
   else
    {
      $oval=$over2;
    }
}

sub sort_csource_probe
{
  $csource_probe{$a} <=> $csource_probe{$b};
}


# --- overview qos and source
$pcount=1;
$overtit="QoS and Source";
&report_over_header2;
if ($overview_sort eq 'd')
 {
   foreach $qsentry (sort sort_cqos_source keys(%cqos_source))
   {
     &sqos_source;
   }
 }
else
 {
   foreach $qsentry (sort keys(%cqos_source))
    {
      &sqos_source;
    }
 }
&report_emptyline_over;

sub sqos_source
{
   $over1=$qsentry;
   $over2=$cqos_source{$qsentry};
   $over0="QoS_Source $pcount";
   if ($qsentry =~ /_ok/i)
    {
      $ook=$over2;
      $onok=$oval;
      $over1=~ s/\_ok//g ;
      &report_over_detail(); 
      $pcount++;
      $ocount++;
    }
   else
    {
      $oval=$over2;
    }
}

sub sort_cqos_source
{
  $cqos_source{$a} <=> $cqos_source{$b};
}

# --- overview probe and qos
$pcount=1;
$overtit="Probe and QoS";
&report_over_header2;
# - in comment sort on number
if ($overview_sort eq 'd')
 {
   foreach $qpentry (sort sort_cprobe_qos keys(%cprobe_qos))
   {
     &sprobe_qos;
   }
 }
else
 {
  foreach $qpentry (sort keys(%cprobe_qos))
   {
     &sprobe_qos;
   }
 }
&report_emptyline_over;

sub sprobe_qos
{
     $over1=$qpentry;
     $over2=$cprobe_qos{$qpentry};
     $over0="Probe_QoS $pcount";
     if ($qpentry =~ /_ok/i)
      {
        $ook=$over2;
        $onok=$oval;
        $over1=~ s/\_ok//g ;
        &report_over_detail(); 
        $pcount++;
        $ocount++;
      }
     else
      {
        $oval=$over2;
      }
}

sub sort_cprobe_qos
{
  $cprobe_qos{$a} <=> $cprobe_qos{$b};
}

# --- fill up the page
while ($teller < 45)
 {
  &report_emptyline;
  $teller = $teller + 1;
 }

# --- footers HTML
&report_end;

# --- close HTML
close (OUTFILE1);

while ($ocount < 30)
 {
   &report_emptyline_over;
   $ocount++;
 }

&report_end_over;
close (OUTFILE2);
close (DELFILE);

print "\n";
print "Created report: $outfil.html\n";
print "Created overview report: $outfil\_overview.html\n";
print "\n";
print "Generated: $countdel delete statements for qos entries not updated the last: $filt_days days\n";
print "\n";
exit 0;

# --- procedures

sub report_header1
 {
  print OUTFILE1 "<html>\n";
  print OUTFILE1 "<meta http-equiv=\"Content-Language\" content=\"en-us\">\n";
  print OUTFILE1 "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1252\">\n";
   print OUTFILE1 "<meta http-equiv=\"REFRESH\" CONTENT=\"$reportseconds\" content=\"no-cache\">\n"; 
  print OUTFILE1 "<title>Nimsoft QOS lastpoll</title>\n";
  print OUTFILE1 "</head>\n";
  print OUTFILE1 "<body>\n";
#  This section is used a header to indicate the type of report and when it was generated
  print OUTFILE1 "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" style=\"border-collapse: collapse\" bordercolor=\"#111111\" width=\"100%\" id=\"AutoNumber1\">\n";
# Put references to other reports
  print OUTFILE1 "<tr>\n";
  print OUTFILE1 "<td width=\"100%\" bgcolor=\"#009AD6\" colspan=\"$colspan\" align=\"left\" height=\"30\"><b>\n";
  print OUTFILE1 "<font color=\"#FFFFFF\" face=\"MS Sans Serif\" size=\"2\">&nbsp;Nimsoft QoS last poll info on $dag $dagnum $maand ($maandnum) $jaar $intim </font>\n";
  print OUTFILE1 "</b></td>\n";
  print OUTFILE1 "</tr>\n";
 }

sub report_over_header1
 {
  if ($overview_sort eq 'd')
   {
     $htext="Sorted by count ok";
   }
  else
   {
     $htext="Sorted by text";
   } 
  print OUTFILE2 "<html>\n";
  print OUTFILE2 "<meta http-equiv=\"Content-Language\" content=\"en-us\">\n";
  print OUTFILE2 "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1252\">\n";
  if ($reportloop eq 'Y')
   {
     print OUTFILE2 "<meta http-equiv=\"REFRESH\" CONTENT=\"$reportseconds\" content=\"no-cache\">\n"; 
   }
  print OUTFILE2 "<title>Nimsoft QoS Lastpoll Overview</title>\n";
  print OUTFILE2 "</head>\n";
  print OUTFILE2 "<body>\n";
#  This section is used a header to indicate the type of report and when it was generated
  print OUTFILE2 "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" style=\"border-collapse: collapse\" bordercolor=\"#111111\" width=\"100%\" id=\"AutoNumber1\">\n";
# Put references to other reports
  print OUTFILE2 "<tr>\n";
  print OUTFILE2 "<td width=\"100%\" bgcolor=\"#009AD6\" colspan=\"$colspan1\" align=\"left\" height=\"30\"><b>\n";
  print OUTFILE2 "<font color=\"#FFFFFF\" face=\"MS Sans Serif\" size=\"2\">&nbsp;Nimsoft QoS Lastpoll Overview Report on $dag $dagnum $maand ($maandnum) $jaar $intim - $htext</font>\n";
  print OUTFILE2 "</b></td>\n";
  print OUTFILE2 "</tr>\n";
 }


sub report_emptyline
 {
  if ($tel eq "0")
   { 
    print OUTFILE1 "<tr>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE1 "</tr>\n";
    $tel = 1;
   }
  else
   {
    print OUTFILE1 "<tr>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE1 "</tr>\n";
    $tel = 0;
   }
 }

sub report_emptyline_over
 {
  if ($tel eq "0")
   { 
    print OUTFILE2 "<tr>\n";
    print OUTFILE2 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE2 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE2 "<td><font face=\"Tahoma\" size=\"2\" color=\"#FFFFFF\">-</font></td>\n";
    print OUTFILE2 "</tr>\n";
    $tel = 1;
   }
  else
   {
    print OUTFILE2 "<tr>\n";
    print OUTFILE2 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE2 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE2 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE2 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE2 "<td bgcolor=\"#D2FFFF\"><pre>\n";
    print OUTFILE2 "<font face=\"Tahoma\" size=\"2\" color=\"#D2FFFF\">-</font></td></pre>\n";
    print OUTFILE2 "</tr>\n";
    $tel = 0;
   }
 }

sub report_end
 {
  &report_fieldex;
  print OUTFILE1 "</table>\n";
  print OUTFILE1 "</body>\n";
  print OUTFILE1 "</html>\n";
  print OUTFILE1 "<p align=\"center\"><font face=\"Arial\" size=\"-2\">Copyright $jaar (generated by $pgm)<br>\n";
  print OUTFILE1 "<a href=\"http://www.ca.com\">CA</a></font></p>\n";
 }

sub report_end_over
 {
  &report_fieldex;
  print OUTFILE2 "</table>\n";
  print OUTFILE2 "</body>\n";
  print OUTFILE2 "</html>\n";
  print OUTFILE2 "<p align=\"center\"><font face=\"Arial\" size=\"-2\">Copyright $jaar (generated by $pgm)<br>\n";
  print OUTFILE2 "<a href=\"http://www.ca.com\">CA</a></font></p>\n";
 }

sub report_header2
 {
# print month title
     print OUTFILE1 "<tr>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;QoS</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Baseline</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Description</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Origin</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Robot</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Source</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Target</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Probe</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Samplerate</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Oldest Poll</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Last Poll</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Min Sample</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Max Sample</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Avg Sample</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;#Samples</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;</b></font></td>\n";
     print OUTFILE1 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;</b></font></td>\n";
     print OUTFILE1 "</tr>\n";
 }

sub report_over_header2
 {
# print month title
     print OUTFILE2 "<tr>\n";
     print OUTFILE2 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;#</b></font></td>\n";
     print OUTFILE2 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;Count (ok/nok)</b></font></td>\n";
     print OUTFILE2 "<td bgcolor=\"#6BC3CE\"><font face=\"Arial\" size=\"2\"><b>&nbsp;$overtit</b></font></td>\n";
     print OUTFILE2 "</tr>\n";
 }

sub report_detail
 {
# - calculate difference between today and last poll (in days)
    ($syy, $smm, $sdd)=split(/-/,$qldate);
    if ($qldate ne '-' && $qldate ne '')
     {
       $stimestamp=timelocal(0,0,0,$sdd,$smm-1,$syy);
       $sdiff=($stimestamp - time()); 
       $sdays=int($sdiff/(60*60*24));
       if ($sdays ne '0')
         {
           $sdays=substr($sdays,1);
         }
#       $sdays=$sdiff/(60*60*24);
     }
    else
     {
       $sdays="-";
     }
#    print "date: dd: $sdd mm: $smm yy: $syy qldate: $qldate sdays: $sdays\n";
    if ($targetlimit ne '')
     {
       $qtarget=substr($qtarget,0,$targetlimit)
     }

#    if ($todayx eq $qldate || $qldate eq "-")
    if ($filt_days > $sdays || $qldate eq "-")
     {
       $col="#000080";
#       print "eq $qname $qldate ($todayx)\n";
       if ($error_only eq 'y')
        {
          goto end_detail;
        }
     }
    else
     {
       $col="#FF0000";
#       print "not eq $qname $qldate ($todayx)\n";
       &dodelete();
       $counterror++;
     }
   $totcol++;
   if ($tel eq "0")
    { 
    print OUTFILE1 "<tr>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$counta $qname</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qbaseline</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qdesc</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qorigin</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qrobot</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qsource</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qtarget</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qprobe</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qsamplerate</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qodate $qotime</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qldate $qltime</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qmin_sample</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qmax_sample</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qavg_sample </font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qcount</font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp; </font></td></pre>\n";
    print OUTFILE1 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp; </font></td></pre>\n";
    print OUTFILE1 "</tr>\n";
    $tel = 1;
    }
   else
    {
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$counta $qname</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qbaseline</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qdesc</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qorigin</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qrobot</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qsource</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qtarget</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qprobe</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qsamplerate</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qodate $qotime</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$qldate $qltime</font></td></pre>\n";
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qmin_sample</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qmax_sample</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qavg_sample</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp;$qcount</font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp; </font></td></pre>\n"; 
    print OUTFILE1 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">&nbsp; </font></td></pre>\n"; 
    print OUTFILE1 "</tr>\n";
    $tel = 0;
    }
    $teller++;
end_detail:
 }

sub report_over_detail
 {
    if ($overview_count eq "d")
      {
        $ocount="$ook \/ $onok";
      }
     else
      {
        $ocount=$ook+$onok;
      } 
    if ($ook eq "0" && $only_view eq "d")
     {
       $col="#FF0000";
     }
    else
     {
       $col="#000080";
     }
   if ($tel eq "0")
    { 
    print OUTFILE2 "<tr>\n";
    print OUTFILE2 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$over0</font></td></pre>\n";
    if ($only_view ne "d")
     {
#       print OUTFILE2 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$onok</font></td></pre>\n";
       print OUTFILE2 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$ocount</font></td></pre>\n";
     }
    else
     {
       print OUTFILE2 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$ocount</font></td></pre>\n";
     }
    print OUTFILE2 "<td><pre><font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$over1</font></td></pre>\n";
    print OUTFILE2 "</tr>\n";
    $tel = 1;
    }
   else
    {
    print OUTFILE2 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE2 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$over0</font></td></pre>\n"; 
    print OUTFILE2 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    if ($only_view ne "d")
     {
#       print OUTFILE2 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$onok</font></td></pre>\n"; 
       print OUTFILE2 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$ocount</font></td></pre>\n"; 
     }
    else
     {
       print OUTFILE2 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$ocount</font></td></pre>\n"; 
     }
    print OUTFILE2 "<td bgcolor=\"#D2FFFF\"><pre>\n"; 
    print OUTFILE2 "<font face=\"Tahoma\" size=\"2\" color=\"$col\">&nbsp;$over1</font></td></pre>\n"; 
    print OUTFILE2 "</tr>\n";
    $tel = 0;
    }
    $teller++;
 }

sub calcdate
 {
  $timeval = time();
  @timelist = gmtime ($timeval);
  if (@timelist[6] eq '0') { $dag = "Sunday" }
  if (@timelist[6] eq '1') { $dag = "Monday" }
  if (@timelist[6] eq '2') { $dag = "Tuesday" }
  if (@timelist[6] eq '3') { $dag = "Wednesday" }
  if (@timelist[6] eq '4') { $dag = "Thursday" }
  if (@timelist[6] eq '5') { $dag = "Friday" }
  if (@timelist[6] eq '6') { $dag = "Saturday" }
  if (@timelist[4] eq '0') { $maand = "January" }
  if (@timelist[4] eq '1') { $maand = "February" }
  if (@timelist[4] eq '2') { $maand = "March" }
  if (@timelist[4] eq '3') { $maand = "April" }
  if (@timelist[4] eq '4') { $maand = "May" }
  if (@timelist[4] eq '5') { $maand = "June" }
  if (@timelist[4] eq '6') { $maand = "July" }
  if (@timelist[4] eq '7') { $maand = "Augustus" }
  if (@timelist[4] eq '8') { $maand = "September" }
  if (@timelist[4] eq '9') { $maand = "October" }
  if (@timelist[4] eq '10') { $maand = "November" }
  if (@timelist[4] eq '11') { $maand = "December" }
  $maandnum = @timelist[4] +1;
  if ($maandnum < 10) {$maandnum="0$maandnum";}
  $dagnum = @timelist[3] ;
  if ($dagnum < 10) {$dagnum="0$dagnum";}
  $jaar = @timelist[5] + 1900;
#
# get time 
#
  $intim1 = time();
  $intim2 = localtime($intim1);
  @intime = split(/ +/ , $intim2);
  $intim = $intime[3]; 

# - format date to be used to compare with last poll (yyyy-mm-dd)
  $todayx="$jaar-$maandnum-$dagnum";


  print ("Today its $dag $dagnum $maand ($maandnum) $jaar $intim \n");
 }

sub report_fieldex
{
# Field explanation
print OUTFILE1 "</table>\n";
print OUTFILE1 "<table>\n";
print OUTFILE1 "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" style=\"border-collapse: collapse\" bordercolor=\"#111111\" width=\"100%\" id=\"AutoNumber1\">\n";
print OUTFILE1 "<tr>\n";
print OUTFILE1 "<td width=\"100%\" bgcolor=\"#009AD6\" colspan=\"$colspan\" align=\"left\" height=\"30\"><b>\n";
print OUTFILE1 "<pre><font color=\"#D2FFFF\" face=\"MS Sans Serif\" size=\"2\">&nbsp;Field explanation ($sqlcount QoS metrics found, $counterror qos flagged as error because they are not polled the last $filt_days days, $countbase matching baseline definitions found)";
print OUTFILE1 "</b></td></pre>\n";
print OUTFILE1 "</tr>\n";

# --- start with blue empty line
$tel=1;
&report_emptyline;
$tel=0;
&report_emptyline; 

print OUTFILE1 "<tr><td bgcolor=\"#D2FFFF\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">QoS:</font></td></pre>\n";
print OUTFILE1 "<td bgcolor=\"#D2FFFF\" colspan=\"$colspan\" align=\"left\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Standard name of the QoS metric</font></td></pre></tr>\n"; 
  
print OUTFILE1 "<tr><td width=\"10%\"><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">Description: </font></td></pre>\n";
print OUTFILE1 "<td width=\"90%\"><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">Standard description for that QoS metric</font></td></pre></tr>\n";

print OUTFILE1 "<tr><td bgcolor=\"#D2FFFF\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Robot:</font></td></pre>\n";
print OUTFILE1 "<td bgcolor=\"#D2FFFF\" colspan=\"$colspan\" align=\"left\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Machine name where the probe is installed</font></td></pre></tr>\n"; 
  
print OUTFILE1 "<tr><td width=\"10%\"><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">Source: </font></td></pre>\n";
print OUTFILE1 "<td width=\"90%\"><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">Monitored machine name</font></td></pre></tr>\n";

print OUTFILE1 "<tr><td bgcolor=\"#D2FFFF\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Probe:</font></td></pre>\n";
print OUTFILE1 "<td bgcolor=\"#D2FFFF\" colspan=\"$colspan\" align=\"left\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Probe name\\id</font></td></pre></tr>\n"; 
  
print OUTFILE1 "<tr><td width=\"10%\"><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">Samplerate: </font></td></pre>\n";
print OUTFILE1 "<td width=\"90%\"><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">Rate that the metrics are polled</font></td></pre></tr>\n";

print OUTFILE1 "<tr><td bgcolor=\"#D2FFFF\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Oldest and Last Poll:</font></td></pre>\n";
print OUTFILE1 "<td bgcolor=\"#D2FFFF\" colspan=\"$colspan\" align=\"left\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Oldest and most recent dates found in the QoS data file</font></td></pre></tr>\n"; 

print OUTFILE1 "<tr><td width=\"10%\"><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">Samples: </font></td></pre>\n";
print OUTFILE1 "<td width=\"90%\"><pre><font face=\"Tahoma\" size=\"2\" color=\"#000080\">Minimum, Maximum and Average sample values found in the QoS data file</font></td></pre></tr>\n";

print OUTFILE1 "<tr><td bgcolor=\"#D2FFFF\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Sample Count:</font></td></pre>\n";
print OUTFILE1 "<td bgcolor=\"#D2FFFF\" colspan=\"$colspan\" align=\"left\"><pre>\n"; 
print OUTFILE1 "<font face=\"Tahoma\" size=\"2\" color=\"#000080\">Number of entries found in the QoS data file</font></td></pre></tr>\n"; 

# --- start with white empty line
$tel=0;
&report_emptyline;
$tel=1;
&report_emptyline;   
}

# ---
sub uim_baseline_list
{
#
# --- Block to read baselines via callback to baseline_engine

$ihub=0;
$robot_count=0;

# --- Login Nimsoft
my $sid = nimLogin($UIM_username, $UIM_password);

# --- get_monitored_metrics
$pds_in = Nimbus::PDS->new();
# print "Callback: get_monitored_metrics on: $baseline_addr\n";
($rc, $pds_data) = doNimNamedRequest($baseline_addr, 'get_monitored_metrics', $pds_in->data(), $request_timeout, $request_retries);
if ($rc)
  {
     print "rc=$rc of nimNamedRequest 'get_monitored_metrics' from hub $hub_addr\n"
  }
$pds_out = Nimbus::PDS->new($pds_data);

# - $pds_out->dump();
doPDS($pds_out);
}

# ---
sub doPDS
{
  my $pds = shift;

  $level++;
  $pds->rewind();
  while(1)
  {
    ($rc, $key, $type, $size, $data) = pdsGetNext($pds->data());
    last if ($rc != NIME_OK);
    $string = $pds_type2string{$type};
#    print("level=$level rc=$rc key=$key type=$string size=$size data=$data\n");
#    print("$level $key $string $size $data\n");
     if ($string eq "PDS_PDS")
      {
        print("$key: $string $size $data\n");
      }
     else
      {
        if ($level eq '1')
         {
           $metcount++;
         }
#        print("   key: $key string: $string size: $size data: $data\n");
        ($i_interval,$i_outliermin,$i_outliermax,$i_nativeinterval,$i_met_id) = split(',', $data);
        ($a_name,$a_met_id) = split(':', $i_met_id);
        $a_met_id=~ tr/"//d;
        $a_met_id=~ tr/}//d;
#        print "$metcount met_id: $a_met_id\n";
# --- add metid in table (so we can use exist to know if a baseline was defined)
        $tab2{$a_met_id}="$a_met_id";

      }
    if ($string eq "PDS_PPDS" || $string eq "PDS_PPCH")
    {
      $pds_data = $pds->get($key, $pds_type{"PDS_PDS"});
      doPDS($pds_data);
    }
    elsif ($string eq "PDS_PDS" )
    {
      $pds_data = Nimbus::PDS->new($data);
      doPDS($pds_data);
    }
  }
  $level--;
#if ($level eq '0')
# {
#   print "Processed $metcount baseline definitions\n";
# }
}

sub doNimNamedRequest
{
    my $address = shift;
    my $command = shift;
    my $data = shift;
    my $timeout = shift;
    my $retries = shift;

    ($rc, $pds_data);
    for (my $itry = 0; $itry < $retries; $itry++)
    {
#        print "nimNamedRequest addr: $address cmd: $command data: $data timeout: $timeout \n";
        ($rc, $pds_data) = nimNamedRequest($address, $command, $data, $timeout);
        last if (! $rc);
        if ($debug == 2) 
         {
           if ($command ne '_status' && $command ne 'list_subscribers' && $command ne 'getrobots' && $command ne 'probe_list')
            {
              print "rc=$rc for nimNamedRequest address=$address command=$command timeout=$timeout try=$itry\n";
            }
         }
    }
    return($rc, $pds_data);
}

sub getparams
{
  usage() unless @ARGV gt 0;
  my %Opts;
  getopts('d:f:u:p:r:y:w:v:8:o:b:s:t:g:e:a:1:2:q:l:',\%Opts);
  $outdir=$Opts{d};
  $outfile=$Opts{f};
  $sqluser=$Opts{u};
  $sqlpass=$Opts{p};
  $sqlopt=$Opts{y};
  $sqlserver=$Opts{r};
  $targetlimit=$Opts{w};
  $only_view=$Opts{v};
  $database=$Opts{8};
  $filt_qos=$Opts{q};
  $filt_probe=$Opts{o};
  $filt_robot=$Opts{b};
  $filt_source=$Opts{s};
  $filt_target=$Opts{t};
  $filt_origin=$Opts{g};
  $filt_days=$Opts{a};
  $error_only=$Opts{e};
  $overview_sort=$Opts{1};
  $overview_count=$Opts{2};
  $report_baseline=$Opts{l};

# --- get credentials and settings from common file (nimsoft_generic.dat)
($Server_With_Rest_Interface, $RestServicePort, $UIM_username, $UIM_password,$UIM_Domain,$UIM_Hub,$UIM_Robot,$sql_server,$sql_user,$sql_password,$sql_db,$sql_type) = nimsoft_generic::getRestData();

#
$baseline_addr="/$UIM_Domain/$UIM_Hub/$UIM_Robot/baseline_engine";
# -- de-crypt (need $encoded)
$decoded = decode_base64($UIM_password);
$UIM_password = RC4($crkey, $decoded);

$decoded = decode_base64($sql_password);
$sql_password = RC4($crkey, $decoded);

# --- if not used in parameters, use defaults from generic
if ($sqluser eq '') { $sqluser=$sql_user; }
if ($sqlpass eq '') { $sqlpass=$sql_password; }
if ($sqlserver eq '') { $sqlserver=$sql_server; }
if ($database eq '') { $database=$sql_db; }


# --- set defaults
  if ($sqlopt eq '1' || $sqlopt eq '')
   {
      $sqltype="ms";
   }
  if ($sqlopt eq '2')
   {
      $sqltype="my";
   }
  if ($sql_type eq 'mssql')
   {
      $sqltype="ms";
   } 
  if ($sql_type eq 'mysql')
   {
      $sqltype="my";
   } 
  if ($sqluser eq '')
   {
     if ($sqltype eq 'my')
      {
        $sqluser = "root";
      }
     if ($sqltype eq 'ms')
      {
        if ($sqlpass eq '')
         {
           $sqluser="trusted";
         }
        else
         {
           $sqluser = "sa";
         }
      }
   }
  if ($sqluser eq 'trusted')
      {
         $trusted=yes;
      }
  if ($sqlpass eq '')
   {
     if ($sqluser eq 'trusted')
      {
         $trusted=yes;
      }
     else
      {
        print "By default we try to access the local mysql with user: $sqluser\n";
        print "BUT you need at least the -p parameter to give a password\n";
        exit 12;
      }
   }
  if ($sqlserver eq '')
   {
     $sqlserver=$hostn;
     if ($sqltype eq 'my')
      {
        print "We will connect the the mysql DB on server: $sqlserver\n";
      }
     if ($sqltype eq 'ms')
      {
        print "We will connect the the mssql DB on server: $sqlserver and userid: $sqluser\n";
      }
   }
  if ($outdir eq '')
   {
     $outdir = "c:\\temp";
   }
  if ($outfile eq '')
   {
     $outfile = "report\_nimsoft\_qos\_lastpoll";
     $delfile = "report\_nimsoft\_qos\_lastpoll\_delete";
   }

  if ($only_view eq '' && ( ( $only_view ne 'y') && ($only_view ne 'd')) )
   {
     $only_view="n";
   }

  if ($database eq '')
   {
     $database="CA_UIM";
   }
  if ($report_baseline eq '')
   {
     $report_baseline="n";
   }
  if ($error_only eq '')
   {
     $error_only="n";
   }
  if ($filt_days eq '')
   {
     $filt_days="3";
   }
  if ($overview_sort eq '')
   {
     $overview_sort="d";
   }
  if ($overview_count eq '')
   {
     $overview_count="t";
   }
  if ($overview_count eq 't')
   {
    $overview_sort="d";
   }
  if ($overview_count eq 'd')
   {
    $overview_sort="n";
    print "Report Overview with ok\/nok sorted on qos name\n";
   }
  if ($overview_sort eq 'd')
   {
     $overview_count="t";
     print "Report Overview on qos totals sorted on qos #\n"; 
   }

# --- create dir + name
$outfil = "$outdir\\$outfile";
$delfil = "$outdir\\$delfile";

%pds_type = (
	"PDS_I", 1,
	"PDS_PI", 2,
	"PDS_PPI", 3,
	"PDS_RGI", 4,
	"PDS_RGPI", 5,
	"PDS_CH", 6,
	"PDS_PCH", 7,
	"PDS_PPCH", 8,
	"PDS_RGCH", 9,
	"PDS_RGPCH", 10,
	"PDS_DUM_L", 11,
	"PDS_DUM_PL", 12,
	"PDS_DUM_PPL", 13,
	"PDS_DUM_RGL", 14,
	"PDS_DUM_RGPL", 15,
	"PDS_F", 16,
	"PDS_PF", 17,
	"PDS_PPF", 18,
	"PDS_RGF", 19,
	"PDS_RGPF", 20,
	"PDS_PDS", 21,
	"PDS_VOID", 22,
	"PDS_SEP", 23,
	"PDS_PPDS", 24,
	"PDS_CPCH", 25,
	"PDS_CPDS", 26,
	"PDS_INT64", 27
);

%pds_type2string = reverse %pds_type;

}


sub dodelete
{
# - delete selected qos entries
#       DELETE from 'RN_QOS_DATA_0014' where table_id='5429'
#       DELETE from 'HN_QOS_DATA_0014' where table_id='5429'
#       DELETE from S_QOS_DATA where table_id ='5429'
     $countdel++;
     print DELFILE "\/* $countdel Qos: $qname Source: $qsource Target: $qtarget Probe: $qprobe LastDate: $qldate *\/ \n";
     print DELFILE "DELETE from $qr_table where table_id='$qtable_id'\n";
     print DELFILE "DELETE from $qh_table where table_id='$qtable_id'\n";
     print DELFILE "DELETE from S_QOS_DATA where table_id='$qtable_id'\n";

}

sub usage
 {
   print "      \n";
   print "nimsoft_qos_lastpoll -v\"y\"\n";
   print "      \n";
   print "  Input parameters:\n";
   print "  -d: output directory (default: c:\\temp)\n";
   print "  -f: output file (default: report_nimsoft_qos_lastpoll)\n";
#   print "  -u: sql userid (default: mysql: root mssql: sa) (trusted: for windows auth.)\n";
#   print "  -p: sql password *** required ***\n";
#   print "  -r: repository/server name: default local host\n";
#   print "  -8: database name, default: CA_UIM\n"; 
#   print "  -y: 1: mssql 2: mysql (default: 1)\n";
   print "      \n";
   print "  Report parameters:\n";
   print "  -w: # of characters used for target column (default: no limit)\n";
   print "  -v: view type:\n";
   print "      y: print on screen probes and # qos\n";
   print "      n: normal view (with no detail query per qos)\n";
   print "      d: detail view (1 extra query per qos to obtain last update)\n";
   print "  -1: overview report sort on name (n) or digits (d) default: d\n";
   print "  -2: overview report count total (t) or ok/nok (d), default: t\n";
   print "  -l: check if baseline definition exist (y,n) default n\n";
   print "  -e: report only error detail records (y ro n) default: n\n";
   print "  -a: #days since lastpoll to report as error, default: 3\n";
   print "      \n";
   print "  Regex filters (Case Insensitive):\n";
   print "  -q: qos name include regex \n";
   print "  -g: origin include regex\n";
   print "  -o: probe include regex \n";
   print "  -b: robot include regex \n";
   print "  -s: source include regex \n";
   print "  -t: target include regex \n";
   exit 0;
}
